require 'spec_helper'

describe Banter::Message do
  let(:original) { {'data'=> 'found'} }
  let(:subject) { Banter::Message.new }
  let(:routing_key) { "test.message" }
  let(:context) { {unique_id: "1234", orig_ip_address: "127.0.0.1"}}

  describe "#serialize" do
    let(:result) { subject.serialize(context, routing_key, original)}

    it "should not fail" do
      expect{ result }.not_to raise_error()
    end

    it "should have all the header items" do
      expect(result[:ts]).not_to be_nil
      expect(result[:ts].class).to eq(Float)
      expect(result[:v]).not_to be_nil
      expect(result[:pub]).not_to be_nil
      expect(result[:payload]).not_to be_nil
    end

    it "should have the data in the payload" do
      expect(result.payload.data).to eq('found')
    end

    it "should have the attributes readable" do
      result
      expect(subject.ts).not_to be_nil
      expect(subject.version).not_to be_nil
      expect(subject.pub).not_to be_nil
      expect(subject.payload).not_to be_nil
    end
  end

  describe "#parse" do
    let(:payload) { subject.serialize(context, routing_key, original).to_json }
    let(:result) { subject.parse(payload) }

    context "standard message" do
      it "should not fail" do
        expect{ result }.not_to raise_error()
      end

      it "should have all the hash items" do
        expect(result[:ts]).not_to be_nil
        expect(result[:v]).not_to be_nil
        expect(result[:pub]).not_to be_nil
        expect(result[:payload]).not_to be_nil
        expect(result[:dead_letter]).to be_nil
        expect(subject.retry_completed?).to eq(false)
      end

      it "should have the data in the payload" do
        expect(result.payload.data).to eq('found')
      end

      it "should have the attributes readable" do
        result
        expect(subject.ts).not_to be_nil
        expect(subject.version).not_to be_nil
        expect(subject.pub).not_to be_nil
        expect(subject.payload).not_to be_nil
        expect(subject.dead_letter).to be_nil
      end
    end

    context "non-banter message" do
      let(:payload) { "non_banter_message" }
      let(:result) { subject.parse(payload) }

      it "should not fail" do
        expect{ result }.not_to raise_error
        expect(result.payload).to eq(payload)
        expect(result.context["format"]).to eq("invalid")
      end


    end

    context "dead lettered message" do
      let(:complete_val) { 1 }
      let(:payload) do
        obj = subject.serialize(context, routing_key, original)
        obj.dead_letter = {"completed"=> complete_val, "id"=> 5}
        obj.to_json
      end

      before do
        result
      end

      context "completed message" do
        it "should have all the hash items" do
          expect(result[:ts]).not_to be_nil
          expect(result[:v]).not_to be_nil
          expect(result[:pub]).not_to be_nil
          expect(result[:payload]).not_to be_nil
          expect(result[:dead_letter]).not_to be_nil
          expect(subject.retry_completed?).to eq(true)
        end
      end

      context "failed message" do
        let(:complete_val) { 0 }
        it "should have all the hash items" do
          expect(result[:ts]).not_to be_nil
          expect(result[:v]).not_to be_nil
          expect(result[:pub]).not_to be_nil
          expect(result[:payload]).not_to be_nil
          expect(result[:dead_letter]).not_to be_nil
          expect(subject.retry_completed?).to eq(false)
        end
      end
    end
  end

end
