#encoding: utf-8
require 'spec_helper'

describe Banter::RabbitLogger do
  let(:routing_key) { "test.logger" }
  let(:subject) { Banter::RabbitLogger }
  let(:payload) { {"hit"=>"me"} }
  let(:message) { Banter::Message.new.serialize(context, routing_key, payload) }
  let(:config_buffer) { StringIO.new }
  let(:output_string) { config_buffer.string }
  let(:delivery_info) { {empty: true}}
  let(:properties) { {empty: true}}
  let(:context) { {unique_id: "1234", orig_ip_address: "127.0.0.1"}}

  context "standard character set" do
    before do
      allow(Banter.logger).to receive(:debug)
      allow(Banter.logger).to receive(:info)
      allow(Banter.logger).to receive(:warn)
      allow(Banter.logger).to receive(:error)
      allow(Banter.logger).to receive(:log)
    end

    describe "#log_publish" do
      let!(:result) { subject.log_publish(routing_key, message)}

      it "should not fail" do
        expect{ result }.not_to raise_error
      end

      it "should write a row to the log" do
        expect(Banter.logger).to have_received(:warn).with(anything)
      end

    end

    describe "#failed_publish" do
      let!(:result) { subject.failed_publish(routing_key, {}, message)}

      context "warning log level" do

        it "should not fail" do
          expect{ result }.not_to raise_error()
        end

        it "should respect the log level of the file" do
          expect(Banter.logger).to have_received(:error).with(anything)
        end
      end
    end

    describe "#log_receive" do
      let!(:result) { subject.log_receive(routing_key, message)}

      it "should not fail" do
        expect{ result }.not_to raise_error()
      end

      it "should respect the log level of the file" do
        expect(Banter.logger).to have_received(:warn).with(anything)
      end
    end

    describe "#log_complete" do
      let!(:result) { subject.log_complete(routing_key, message)}

      it "should not fail" do
        expect{ result }.not_to raise_error()
      end

      it "should respect the log level of the file" do
        expect(Banter.logger).to have_received(:warn).with(anything)
      end
    end

    describe "#log" do
      let!(:result) { subject.log(::Logger::DEBUG, message)}

      it "should not fail" do
        expect{ result }.not_to raise_error()
      end

      it "should respect the log level of the file" do
        expect(Banter.logger).to have_received(:debug).with(anything)
      end
    end

    describe "#log_service" do
      let(:service_name) { "test" }
      let!(:result) { subject.log_service(service_name, message)}

      context "debugger log" do
        it "should not fail" do
          expect{ result }.not_to raise_error()
        end

        it "should respect the log level of the file" do
          expect(Banter.logger).to have_received(:info).with(anything)
        end
      end
    end

    describe "#log_subscriber_failed" do
      let(:contents) { {"data"=>"filler text"} }
      context "message is badly set up" do
        let(:message) { nil }
        let!(:result) { subject.log_subscriber_failed(routing_key, delivery_info, properties, message, contents ) }

        it "should not fail" do
          expect{ result }.not_to raise_error()
        end

      end

      context "message is correct" do
        let!(:result) { subject.log_subscriber_failed(routing_key, delivery_info, properties, message, contents )}

        it "should not fail" do
          expect{ result }.not_to raise_error()
        end

        it "should respect the log level of the file" do
          expect(Banter.logger).to have_received(:warn).with(anything)
        end
      end

    end
  end

end
