require 'spec_helper'

# There isn't a good way to stub and mock methods in classes that include Celluloid
# The only workaround I have right now, is to check other methods that get called

describe Banter::Server::SubscriberServer do
  subject { Banter::Server::SubscriberServer.instance }
  let!(:application_name) { "testsubscriber" }
  let!(:durable) { true }

  before do
    Banter::Configuration.application_name = application_name
  end

  describe '#set_workers' do
    context "default config" do
      before {
        allow(Banter::Server::ClientQueueListener).to receive(:new)
        subject.set_workers([MyTestSubscriber1, MyTestSubscriber2])
      }

      it "creates queue listeners" do
        expect(Banter::Server::ClientQueueListener).to have_received(:new).with(MyTestSubscriber1)
        expect(Banter::Server::ClientQueueListener).to have_received(:new).with(MyTestSubscriber2)
      end
    end

    context "set pool_size" do
      before {
        Banter::Configuration.application_name = application_name
        @default_pool_size = Banter::Configuration.configuration[:pool_size]
        allow(Banter::Server::ClientQueueListener).to receive(:new)
        Banter::Configuration.configuration[:pool_size] = 200
        subject.set_workers([MyTestSubscriber1, MyTestSubscriber2])
      }

      after {
        Banter::Configuration.configuration[:pool_size] = @default_pool_size
      }

      it "creates queue listeners" do
        expect(Banter::Server::ClientQueueListener).to have_received(:new).with(MyTestSubscriber1)
        expect(Banter::Server::ClientQueueListener).to have_received(:new).with(MyTestSubscriber2)
      end

    end
  end

  describe '#set_information' do
    before { subject.send(:set_information)}
    it { expect(subject.process_name).not_to be_nil }
    it { expect(subject.pid).not_to be_nil }
    it { expect(subject.hostname).not_to be_nil }
  end

end